#ifndef GAME_H
#define GAME_H

#include <system/application.h>
#include <graphics/sprite.h>
#include <graphics/sprite_renderer.h>
#include <graphics/font.h>
#include <input/vita/controller_input_manager_vita.h>
#include <vector>
#include <system/platform.h>
#include <iostream>
#include <math.h>

#include "player.h"
#include "weapon.h"
#include "map.h"
#include "gameobject.h"

//define game screens 
const int StartScreen = 1,
		  GameOverScreen = 2,
		  PauseScreen = 3,
		  GameScreen = 4,
		  WinScreen = 5;
// define text properties
const float TextXCoordinate = 0.f,
			TextYCoordinate = 272.f,
			TextOffset = 30.f,
			HealthDisplayXCoordinate = 10.f,
			HealthDisplayYCoordinate = 80.f,
			MessageSize = 1.5f,
			ScreenTextSize = 2.f,
			HealthDisplaySize = 1.f;
const UInt32 TextColour = 0xffffffff,
			 HealthDisplayColour = 0xff00ff00;

namespace abfw
{
	class Platform;
}

class Game : public abfw::Application
{	
 public:
	Game(abfw::Platform* platform);
	void Init(), // initialises the game
		 CleanUp(), // removes the sprite renderer when the application closes
		 Render(), // renders sprites and text
		 DrawOnScreen(GameObject& Object), // draws all objects on screen
		 ChangeGameScreen(); // switches between start screen, game screen, gameover screen and pause screen
	bool Update(float ticks); // main game loop

 private:
	Player MyPlayer; // create the player
	Weapon PlayerWeapon; // create the player's weapon
	Map MyMap; // create the map
	GameObject MyCamera; // create the camera
	float frame_rate_; // number of frames displayed per second
	int frame_count_, // number of frames that have passed since the game started
		current_game_screen_; // stores the current game screen
	bool impassable_door_; // used to let the player know when a door is impassable
	abfw::ControllerInputManagerVita controller_manager_; // takes input from the user
	abfw::Font font_; // used to display text on screen
	abfw::SpriteRenderer* sprite_renderer_; // renders all the sprites
};

#endif
