#include "gameobject.h"

GameObject::GameObject()
{
	// make any game object messages not seen by default
	set_message_seen(false);
}

void GameObject::MoveTo(float xposition, float yposition, float zposition)
{
	// move the game object to a specified position
	set_position(abfw::Vector3(xposition, yposition, zposition));
}

void GameObject::SetX(float xposition)
{
	// change the game object's x coordinate
	set_position(abfw::Vector3(xposition, position().y, position().z));
}

void GameObject::SetY(float yposition)
{
	// change the game object's y coordinate
	set_position(abfw::Vector3(position().x, yposition, position().z));
}

void GameObject::SetZ(float zposition)
{
	// change the game object's z coordinate
	set_position(abfw::Vector3(position().x, position().y, zposition));
}

void GameObject::MoveDeltaX(float xdistance)
{
	// move the game object in the x axis
	set_position(abfw::Vector3(position().x + xdistance, position().y, position().z));
}

void GameObject::MoveDeltaY(float ydistance)
{
	// move the game object in the y axis
	set_position(abfw::Vector3(position().x, position().y + ydistance, position().z));
}

void GameObject::MoveDeltaZ(float zdistance)
{
	// move the game object in the z axis
	set_position(abfw::Vector3(position().x, position().y, position().z + zdistance));
}

bool GameObject::CollisionDetection(GameObject& Object)
{
	// get the positions and half sizes of the two game objects
	float xposition1 = position().x,
		  yposition1 = position().y,
		  xposition2 = Object.position().x,
		  yposition2 = Object.position().y,
		  half_width1 = HalfSize(width()),
		  half_height1 = HalfSize(height()),
		  half_width2 = HalfSize(Object.width()),
		  half_height2 = HalfSize(Object.height());

	// see if the first game object is to the left of the second game object
	if((xposition1 + half_width1) < (xposition2 - half_width2))
		return false;
	// see if the first game object is to the right of the second game object
	else if((xposition1 - half_width1) > (xposition2 + half_width2))
		return false;
	// see if the first game object is above the second game object
	else if((yposition1 + half_height1) < (yposition2 - half_height2))
		return false;
	// see if the first game object is below the second game object
	else if((yposition1 - half_height1) > (yposition2 + half_height2))
		return false;
	// if none of the previous conditions are true then the game objects must be colliding
	else
		return true;
}

float GameObject::HalfSize(float size)
{
	// divide the size by two
	size = size * Half;

	return size;
}

void GameObject::UpdateEnemy(float player_xposition, float player_yposition)
{
	// get the horizontal and vertical distances between the enemy and the player
	set_xdistance_from_player(player_xposition - position().x);
	set_ydistance_from_player(player_yposition - position().y);
	// use trigonometry to make the enemy face the player
	set_orientation(atan2(ydistance_from_player(), xdistance_from_player()));
	set_rotation(orientation());
	FlyForwards();
}

void GameObject::FlyForwards()
{
	// use trigonometry to make the game object move in the direction it's facing
	MoveDeltaX(cos(orientation()) * speed());
	MoveDeltaY(sin(orientation()) * speed());
}
