#ifndef GAMEOBJECT_H
#define GAMEOBJECT_H

#include <system/application.h>
#include <graphics/sprite.h>
#include <graphics/sprite_renderer.h>
#include <math.h>

// define the fraction, half
const float Half = 0.5f;
// define enemy properties
const UInt32 EnemyColour = 0xff07a0a0,
			 DamagedColour = 0xff70128c;
const float EnemyWidth = 20.f,
			EnemyHeight = 15.f,
			EnemySpeed = 2.f,
			EnemyHealth = 20.f,
			EnemyDamage = 2.f;
// define platform properties
const UInt32 PlatformColour = 0xff11a600;
const float PlatformHeight = 20.f,
			PlatformZCoordinate = 0.3f;
// define door properties
const UInt32 DoorColour = 0xffffffff;
const float DoorWidth = 20.f,
			DoorHeight = 50.f,
			DoorOffset = 25.f,
			ExitDistance = 40.f,
			DoorZCoordinate = 0.4f;
class GameObject : public abfw::Sprite
{
 public:
	GameObject();  // initialises game object variables
	// getters and setters
	inline void set_velocity(const abfw::Vector2& velocity) { velocity_ = velocity; }
	inline const abfw::Vector2& velocity() const {return velocity_; }
	inline void set_orientation(const float orientation) { orientation_ = orientation; }
	inline const float orientation() const {return orientation_; }
	inline void set_speed(const float speed) { speed_ = speed; }
	inline const float speed() const {return speed_; }
	inline void set_health(const float health) { health_ = health; }
	inline const float health() const {return health_; }
	inline void set_message_seen(const bool message_seen) { message_seen_ = message_seen; }
	inline const bool message_seen() const {return message_seen_; }
	inline void set_xdistance_from_player(const float xdistance_from_player) { xdistance_from_player_ = xdistance_from_player; }
	inline const float xdistance_from_player() const {return xdistance_from_player_; }
	inline void set_ydistance_from_player(const float ydistance_from_player) { ydistance_from_player_ = ydistance_from_player; }
	inline const float ydistance_from_player() const {return ydistance_from_player_; }
	inline void set_room_destination(const int room_destination) { room_destination_ = room_destination;}
	inline const int room_destination() const {return room_destination_; }
	inline void set_keys_needed(const int keys_needed) { keys_needed_ = keys_needed;}
	inline const int keys_needed() const {return keys_needed_; }
	inline void set_new_door(const abfw::Vector2& new_door) { new_door_ = new_door; }
	inline const abfw::Vector2& new_door() const {return new_door_; }
	inline void set_door_exit(const float door_exit) { door_exit_ = door_exit; }
	inline const float door_exit() const {return door_exit_; }
	void MoveTo(float xposition, float yposition, float zposition), // move the game object to a specified position
		 SetX(float xposition), // change the game object's x coordinate
		 SetY(float yposition), // change the game object's y coordinate
		 SetZ(float zposition), // change the game object's z coordinate
		 MoveDeltaX(float xdistance), // move the game object in the x axis
		 MoveDeltaY(float ydistance), // move the game object in the y axis
		 MoveDeltaZ(float zdistance), // move the game object in the z axis
		 UpdateEnemy(float player_xposition, float player_yposition), // moves the enemy towards the player
		 FlyForwards(); // makes the game object fly forward in the direction it's facing
	bool CollisionDetection(GameObject& Object); // check for a collision between 2 game objects
	float HalfSize(float size); // get half of an object's size

 private:
	abfw::Vector2 velocity_, // the velocity of the game object
				  new_door_; // position of the door in the new room
	float orientation_, // the direction the game object is facing
		  speed_, // the speed the game object moves at
		  health_, // how much health the game object has
		  xdistance_from_player_, // horizontal distance between the enemy and the player
		  ydistance_from_player_, // vertical distance between the enemy and the player
		  door_exit_; // position the player exits the door at
	int room_destination_, // room that the door leads to
		keys_needed_;  // number of keys that the door needs to open
	bool message_seen_; // wether or not player has read the game object's message
};

#endif
