#include "map.h"

Map::Map()
{
	// initialise the position of the map
	MoveTo(0, 0, 1);
	// start the game in the main room
	set_current_room(MainRoom);
	// set up the room sizes
	set_width(RoomWidth);
	set_height(RoomHeight);
}

void Map::ChangeRoom()
{
	// delete all the rooms' game objects
	ClearRoom(Platforms);
	ClearRoom(Doors);
	ClearRoom(Enemies); 

	// create the floor
	CreatePlatform(RoomLeft, RoomBottom, width());

	// change the room based on the current room variable
	switch(current_room_)
	{
		case MainRoom:
			// set the colour of the room
			set_colour(StandardColour);
			// create the platforms
			CreatePlatform(RoomLeft + TraversableGap, RoomBottom - TraversableGap, VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 3), RoomBottom - TraversableGap, VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 5), RoomBottom - TraversableGap, VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 7), RoomBottom - TraversableGap, VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 9), RoomBottom - TraversableGap, VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 11), RoomBottom - TraversableGap, VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 13), RoomBottom - TraversableGap, VerySmallPlatform);
			CreatePlatform(RoomLeft, RoomBottom - (TraversableGap * 2), MediumPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 4), RoomBottom - (TraversableGap * 2), MediumPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 8), RoomBottom - (TraversableGap * 2), MediumPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 12), RoomBottom - (TraversableGap * 2), MediumPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 1), RoomBottom - (TraversableGap * 3), SmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 5), RoomBottom - (TraversableGap * 3), VeryLargePlatfrom);
			CreatePlatform(RoomLeft + (TraversableGap * 12), RoomBottom - (TraversableGap * 3), SmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 2), RoomBottom - (TraversableGap * 4), LargePlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 9), RoomBottom - (TraversableGap * 4), LargePlatform);
			CreatePlatform(RoomLeft + TraversableGap, RoomBottom - (TraversableGap * 5), VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 13), RoomBottom - (TraversableGap * 5), VerySmallPlatform);
			CreatePlatform(RoomLeft, RoomBottom - (TraversableGap * 6), VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 14), RoomBottom - (TraversableGap * 6), VerySmallPlatform);
			// create the doors
			CreateDoor(RoomRight, RoomBottom, StickRoom, abfw::Vector2(RoomLeft, RoomBottom), ExitDistance, NoKeys);
			CreateDoor(RoomLeft, RoomBottom, FirstHallway, abfw::Vector2(RoomRight, RoomTop + TraversableGap), -ExitDistance, NoKeys);
			CreateDoor(RoomRight, RoomTop + (TraversableGap * 2), SecondHallway, abfw::Vector2(RoomLeft, RoomBottom), ExitDistance, OneKey);
			CreateDoor(RoomLeft, RoomTop + (TraversableGap * 2), ArmourRoom, abfw::Vector2(RoomRight, RoomBottom), -ExitDistance, NoKeys);
			break;
		case FirstHallway:
			// set the colour of the room
			set_colour(StandardColour);
			// create the platforms
			CreatePlatform(RoomLeft + (TraversableGap * 3), RoomBottom - TraversableGap, MediumPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 14), RoomBottom - TraversableGap, VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 7), RoomBottom - (TraversableGap * 2), SmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 14), RoomBottom - (TraversableGap * 2), VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 3), RoomBottom - (TraversableGap * 3), MediumPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 14), RoomBottom - (TraversableGap * 3), VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 7), RoomBottom - (TraversableGap * 4), SmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 14), RoomBottom - (TraversableGap * 4), VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 3), RoomBottom - (TraversableGap * 5), MediumPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 7), RoomBottom - (TraversableGap * 6), LargePlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 12), RoomBottom - (TraversableGap * 7), MediumPlatform);
			// create the doors
			CreateDoor(RoomRight, RoomTop + TraversableGap, MainRoom, abfw::Vector2(RoomLeft, RoomBottom), ExitDistance, NoKeys);
			CreateDoor(RoomRight, RoomTop + (TraversableGap * 4), SecondKeyRoom, abfw::Vector2(RoomLeft, RoomBottom), ExitDistance, NoKeys);
			CreateDoor(RoomLeft, RoomBottom, GunUpgradeRoom, abfw::Vector2(RoomRight, RoomBottom), -ExitDistance, TwoKeys);
			// create the enemies
			CreateEnemy(RoomLeft + (TraversableGap * 9), RoomBottom - (TraversableGap * 7));
			CreateEnemy(RoomLeft + (TraversableGap * 4), RoomBottom - (TraversableGap * 6));
			CreateEnemy(RoomLeft + (TraversableGap * 8), RoomBottom - (TraversableGap * 5));
			CreateEnemy(RoomLeft + (TraversableGap * 4), RoomBottom - (TraversableGap * 4));
			CreateEnemy(RoomLeft + (TraversableGap * 8), RoomBottom - (TraversableGap * 3));
			CreateEnemy(RoomLeft + (TraversableGap * 4), RoomBottom - (TraversableGap * 2));
			CreateEnemy(RoomLeft + (TraversableGap * 14), RoomBottom - (TraversableGap * 2));
			CreateEnemy(RoomLeft + (TraversableGap * 14), RoomBottom - TraversableGap);
			break;
		case SecondHallway:
			// set the colour of the room
			set_colour(StandardColour);
			// create the platforms
			CreatePlatform(RoomLeft + (TraversableGap * 2), RoomBottom - TraversableGap, VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 4), RoomBottom - (TraversableGap * 2), VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 6), RoomBottom - (TraversableGap * 3), VeryLargePlatfrom);
			CreatePlatform(RoomLeft, RoomBottom - (TraversableGap * 4), VeryLargePlatfrom);
			CreatePlatform(RoomLeft + (TraversableGap * 12), RoomBottom - (TraversableGap * 4), VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 10), RoomBottom - (TraversableGap * 5), VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 12), RoomBottom - (TraversableGap * 6), VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 14), RoomBottom - (TraversableGap * 7), VerySmallPlatform);
			// create the doors
			CreateDoor(RoomLeft, RoomBottom, MainRoom, abfw::Vector2(RoomRight, RoomTop + (TraversableGap * 2)), -ExitDistance, NoKeys);
			CreateDoor(RoomLeft, RoomTop + (TraversableGap * 4), LastRoom, abfw::Vector2(RoomRight, RoomBottom), -ExitDistance, NoKeys);
			CreateDoor(RoomRight, RoomTop + TraversableGap, SwordRoom, abfw::Vector2(RoomLeft, RoomBottom), ExitDistance, ThreeKeys);			
			// create the enemies
			CreateEnemy(RoomLeft + (TraversableGap * 2), RoomBottom - (TraversableGap * 2));
			CreateEnemy(RoomLeft + (TraversableGap * 4), RoomBottom - (TraversableGap * 3));
			CreateEnemy(RoomLeft + (TraversableGap * 8), RoomBottom - (TraversableGap * 4));
			CreateEnemy(RoomLeft + (TraversableGap * 10), RoomBottom - (TraversableGap * 4));
			CreateEnemy(RoomLeft + (TraversableGap * 2), RoomBottom - (TraversableGap * 5));
			CreateEnemy(RoomLeft + (TraversableGap * 12), RoomBottom - (TraversableGap * 5));
			CreateEnemy(RoomLeft + (TraversableGap * 14), RoomBottom - (TraversableGap * 6));
			CreateEnemy(RoomLeft + (TraversableGap * 13), RoomBottom - (TraversableGap * 7));
			break;
		case ArmourRoom:
			// set the colour of the room
			set_colour(UpgradeColour);
			// create the platforms
			CreatePlatform(RoomLeft + (TraversableGap * 6), RoomBottom - TraversableGap, VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 5), RoomBottom - (TraversableGap * 2), VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 4), RoomBottom - (TraversableGap * 3), VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 3), RoomBottom - (TraversableGap * 4), VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 2), RoomBottom - (TraversableGap * 5), VerySmallPlatform);
			CreatePlatform(RoomLeft + TraversableGap, RoomBottom - (TraversableGap * 6), VerySmallPlatform);
			CreatePlatform(RoomLeft, RoomBottom - (TraversableGap * 7), VerySmallPlatform);
			// create the doors
			CreateDoor(RoomRight, RoomBottom, MainRoom, abfw::Vector2(RoomLeft, RoomTop + (TraversableGap * 2)), ExitDistance, NoKeys);
			CreateDoor(RoomLeft, RoomBottom, FirstKeyRoom, abfw::Vector2(RoomRight, RoomBottom), -ExitDistance, NoKeys);
			// create the enemies
			CreateEnemy(RoomLeft + TraversableGap, RoomBottom - (TraversableGap * 7));
			CreateEnemy(RoomLeft + (TraversableGap * 2), RoomBottom - (TraversableGap * 7));
			CreateEnemy(RoomLeft + (TraversableGap * 3), RoomBottom - (TraversableGap * 6));
			CreateEnemy(RoomLeft + (TraversableGap * 3), RoomBottom - (TraversableGap * 5));
			CreateEnemy(RoomLeft + (TraversableGap * 4), RoomBottom - (TraversableGap * 5));
			// create the items and initialise their properties
			Armour.MoveTo(RoomLeft + ItemDistance, RoomBottom - (TraversableGap * 7) - ItemHeight, 0);
			Armour.set_colour(ArmourColour);
			Armour.set_width(ArmourWidth);
			Armour.set_height(ArmourHeight);
			break;
		case StickRoom:
			// set the colour of the room
			set_colour(UpgradeColour);
			// create the platforms
			CreatePlatform(RoomLeft + (TraversableGap * 5), RoomBottom - TraversableGap, VeryLargePlatfrom);
			CreatePlatform(RoomLeft + (TraversableGap * 2), RoomBottom - (TraversableGap * 2), SmallPlatform);
			CreatePlatform(RoomLeft, RoomBottom - (TraversableGap * 3), VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 2), RoomBottom - (TraversableGap * 4), SmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 5), RoomBottom - (TraversableGap * 5), VeryLargePlatfrom);
			CreatePlatform(RoomLeft + (TraversableGap * 10), RoomBottom - (TraversableGap * 4), MediumPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 14), RoomBottom - (TraversableGap * 4), VerySmallPlatform);
			// create the doors
			CreateDoor(RoomLeft, RoomBottom, MainRoom, abfw::Vector2(RoomRight, RoomBottom), -ExitDistance, NoKeys);
			CreateDoor(RoomRight, RoomTop + (TraversableGap * 4), ThirdKeyRoom, abfw::Vector2(RoomLeft, RoomBottom), ExitDistance, NoKeys);
			// create the enemies
			CreateEnemy(RoomLeft + (TraversableGap * 6), RoomBottom - (TraversableGap * 6));
			CreateEnemy(RoomLeft + (TraversableGap * 7), RoomBottom - (TraversableGap * 6));
			CreateEnemy(RoomLeft + (TraversableGap * 8), RoomBottom - (TraversableGap * 6));
			CreateEnemy(RoomLeft + (TraversableGap * 9), RoomBottom - (TraversableGap * 6));
			// create the items and initialise their properties
			Stick.MoveTo(RoomLeft + (TraversableGap * 7) + ItemDistance, RoomBottom - TraversableGap - ItemHeight, 0);
			Stick.set_colour(StickColour);
			Stick.set_width(StickWidth);
			Stick.set_height(WeaponHeight);
			break;
		case FirstKeyRoom:
			// set the colour of the room
			set_colour(KeyRoomColour);
			// create the doors
			CreateDoor(RoomRight, RoomBottom, ArmourRoom, abfw::Vector2(RoomLeft, RoomBottom), ExitDistance, NoKeys);
			// create the items and initialise their properties
			Keys[0].MoveTo(RoomRight - TraversableGap - ItemDistance, RoomBottom - ItemHeight, 0);
			Keys[0].set_colour(KeyColour);
			Keys[0].set_width(KeyWidth);
			Keys[0].set_height(KeyHeight);
			break;
		case SecondKeyRoom:
			// set the colour of the room
			set_colour(KeyRoomColour);
			// create the doors
			CreateDoor(RoomLeft, RoomBottom, FirstHallway, abfw::Vector2(RoomRight, RoomTop + (TraversableGap * 4)), -ExitDistance, NoKeys);			
			// create the items and initialise their properties
			Keys[1].MoveTo(RoomLeft + TraversableGap + ItemDistance, RoomBottom - ItemHeight, 0);
			Keys[1].set_colour(KeyColour);
			Keys[1].set_width(KeyWidth);
			Keys[1].set_height(KeyHeight);
			break;
		case ThirdKeyRoom:
			// set the colour of the room
			set_colour(KeyRoomColour);
			// create the doors
			CreateDoor(RoomLeft, RoomBottom, StickRoom, abfw::Vector2(RoomRight, RoomTop + (TraversableGap * 4)), -ExitDistance, NoKeys);
			// create the items and initialise their properties
			Keys[2].MoveTo(RoomLeft + TraversableGap + ItemDistance, RoomBottom - ItemHeight, 0);
			Keys[2].set_colour(KeyColour);
			Keys[2].set_width(KeyWidth);
			Keys[2].set_height(KeyHeight);
			break;
		case GunUpgradeRoom:
			// set the colour of the room
			set_colour(UpgradeColour);
			// create the doors
			CreateDoor(RoomRight, RoomBottom, FirstHallway, abfw::Vector2(RoomLeft, RoomBottom), ExitDistance, NoKeys);
			// create the items and initialise their properties
			GunUpgrade.MoveTo(RoomRight - TraversableGap - ItemDistance, RoomBottom - ItemHeight, 0);
			GunUpgrade.set_colour(GunColour);
			GunUpgrade.set_width(GunWidth);
			GunUpgrade.set_height(WeaponHeight);
			break;
		case SwordRoom:
			// set the colour of the room
			set_colour(UpgradeColour);
			// create the doors
			CreateDoor(RoomLeft, RoomBottom, SecondHallway, abfw::Vector2(RoomRight, RoomTop + TraversableGap), -ExitDistance, NoKeys);
			// create the items and initialise their properties
			Sword.MoveTo(RoomLeft + TraversableGap + ItemDistance, RoomBottom - ItemHeight, 0);
			Sword.set_colour(SwordColour);
			Sword.set_width(SwordWidth);
			Sword.set_height(WeaponHeight);
			break;
		case LastRoom:
			// set the colour of the room
			set_colour(LastColour);
			// create the platforms
			CreatePlatform(RoomLeft + (TraversableGap * 6), RoomBottom - TraversableGap, VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 5), RoomBottom - (TraversableGap * 2), VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 4), RoomBottom - (TraversableGap * 3), VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 3), RoomBottom - (TraversableGap * 4), VerySmallPlatform);
			CreatePlatform(RoomLeft + (TraversableGap * 2), RoomBottom - (TraversableGap * 5), VerySmallPlatform);
			CreatePlatform(RoomLeft + TraversableGap, RoomBottom - (TraversableGap * 6), VerySmallPlatform);
			CreatePlatform(RoomLeft, RoomBottom - (TraversableGap * 7), VerySmallPlatform);
			// create the doors
			CreateDoor(RoomRight, RoomBottom, SecondHallway, abfw::Vector2(RoomLeft, RoomTop + (TraversableGap * 4)), ExitDistance, NoKeys);
			// create the enemies
			CreateEnemy(RoomLeft + (TraversableGap * 2), RoomBottom - (TraversableGap * 7));
			CreateEnemy(RoomLeft + (TraversableGap * 3), RoomBottom - (TraversableGap * 7));
			CreateEnemy(RoomLeft + (TraversableGap * 4), RoomBottom - (TraversableGap * 7));
			CreateEnemy(RoomLeft + (TraversableGap * 5), RoomBottom - (TraversableGap * 7));
			CreateEnemy(RoomLeft + (TraversableGap * 6), RoomBottom - (TraversableGap * 6));
			CreateEnemy(RoomLeft + (TraversableGap * 7), RoomBottom - (TraversableGap * 6));
			CreateEnemy(RoomLeft + (TraversableGap * 8), RoomBottom - (TraversableGap * 6));
			CreateEnemy(RoomLeft + (TraversableGap * 9), RoomBottom - (TraversableGap * 6));
			CreateEnemy(RoomLeft + (TraversableGap * 2), RoomBottom - (TraversableGap * 5));
			CreateEnemy(RoomLeft + (TraversableGap * 3), RoomBottom - (TraversableGap * 5));
			CreateEnemy(RoomLeft + (TraversableGap * 4), RoomBottom - (TraversableGap * 5));
			CreateEnemy(RoomLeft + (TraversableGap * 5), RoomBottom - (TraversableGap * 5));
			CreateEnemy(RoomLeft + (TraversableGap * 6), RoomBottom - (TraversableGap * 4));
			CreateEnemy(RoomLeft + (TraversableGap * 7), RoomBottom - (TraversableGap * 4));
			CreateEnemy(RoomLeft + (TraversableGap * 8), RoomBottom - (TraversableGap * 4));
			CreateEnemy(RoomLeft + (TraversableGap * 9), RoomBottom - (TraversableGap * 4));
			Objective.MoveTo(RoomLeft + ItemDistance, RoomBottom - (TraversableGap * 7) - ItemHeight, 0);
			Objective.set_colour(ObjectiveColour);
			Objective.set_width(ObjectiveWidth);
			Objective.set_height(ObjectiveHeight);
			break;
	}
}

void Map::ClearRoom(std::vector<GameObject>& objects_to_be_deleted)
{
	// delete every game object in the vector that was passed to the function
	std::vector<GameObject>::iterator object = objects_to_be_deleted.begin();
	while(object != objects_to_be_deleted.end())
	{
		objects_to_be_deleted.erase(object);
	}
}

void Map::CreatePlatform(float platform_start, float platform_level, float width)
{
	// create a new game object for the platform
	GameObject platform;
	// initialises platform variables
	platform.set_colour(PlatformColour);
	platform.set_width(width);
	platform.set_height(PlatformHeight);
	platform.MoveTo(platform_start + HalfSize(width), platform_level, PlatformZCoordinate);
	// add the new platform to the vector of platforms
	Platforms.push_back(platform);
}

void Map::CreateDoor(float xposition, float yposition, int room, abfw::Vector2 other_door, float exit, int required_keys)
{
	// create a new game object for the door
	GameObject door;
	// initialise door properties
	door.set_room_destination(room);
	door.set_keys_needed(required_keys);
	door.set_new_door(abfw::Vector2(other_door.x, other_door.y - DoorOffset));
	door.set_door_exit(exit);
	door.set_colour(DoorColour);
	door.set_width(DoorWidth);
	door.set_height(DoorHeight);
	door.MoveTo(xposition, yposition - HalfSize(door.height()), DoorZCoordinate);
	// add the new door to the vector of doors
	Doors.push_back(door);
}

void Map::CreateEnemy(float xposition, float yposition)
{
	// create a new game object for the enemy
	GameObject enemy;
	// initialise enemy properties
	enemy.set_colour(EnemyColour);
	enemy.set_width(EnemyWidth);
	enemy.set_height(EnemyHeight);
	enemy.set_speed(EnemySpeed);
	enemy.set_health(EnemyHealth);
	enemy.MoveTo(xposition, yposition, 0);
	// add the new enemy to the vector of enemies
	Enemies.push_back(enemy);
}
