#ifndef MAP_H
#define MAP_H

#include <graphics/sprite.h>
#include <graphics/sprite_renderer.h>
#include <iostream>
#include <vector>

#include "player.h"
#include "weapon.h"

// define platform sizes
const float VerySmallPlatform = 100.f,
			SmallPlatform = 200.f,
			MediumPlatform = 300,
			LargePlatform = 400,
			VeryLargePlatfrom = 500;
// define room sizes and positions
const float RoomWidth = 1500.f,
			RoomHeight = 800.f,
			RoomLeft = -750.f, // position of the room's left wall
		    RoomRight = 750.f, // position of the room's right wall
		    RoomTop = -400.f, // position of the room's ceiling
		    RoomBottom = 400.f; // position of the room's floor
// define room numbers
const int MainRoom = 1,
		  FirstHallway = 2,
		  SecondHallway = 3,
		  ArmourRoom = 4,
		  StickRoom = 5,
		  FirstKeyRoom = 6,
		  SecondKeyRoom = 7,
		  ThirdKeyRoom = 8,
		  GunUpgradeRoom = 9,
		  SwordRoom = 10,
		  LastRoom = 11;
//define room colours
const UInt32 StandardColour = 0xfff75f1e,
			 KeyRoomColour = 0xff410daf,
			 UpgradeColour = 0xff004a0f,
			 LastColour = 0xff7500a3;
// define distances
const float TraversableGap = 100.f,
			ItemDistance = 50.f,
			ItemHeight = 24.f;
// define number of keys
const int NoKeys = 0,
		  OneKey = 1,
		  TwoKeys = 2,
		  ThreeKeys = 3;

class Map : public GameObject
{
 public:
	Map();  // intialises map variables
	// vectors for different types of game objects
	std::vector<GameObject> Platforms;
	std::vector<GameObject> Doors;
	std::vector<GameObject> Enemies;
	// items the player can pick up
	GameObject  Keys[3],
				Armour,
				Stick,
				GunUpgrade,
				Sword,
				Objective;
	// getter and setter
	inline void set_current_room(const int current_room) { current_room_ = current_room;}
	inline const int current_room() const {return current_room_; }
	void ChangeRoom(), // puts the player in a new room
		 ClearRoom(std::vector<GameObject>& objects_to_be_deleted), // clear a room of one kind of game object
		 CreateEnemy(float xposition, float yposition), // initialises enemy variables
		 CreatePlatform(float platform_start, float platform_level, float width),  // initialises platform variables
		 CreateDoor(float xposition, float yposition, int room, abfw::Vector2 other_door, float exit, int required_keys); // initialises door variables

 private: 
	 int current_room_; // room the player is currently in
};

#endif
