#include "player.h"

Player::Player()
{
	// initialise player properties
	set_colour(PlayerColour);
	set_width(PlayerWidth);
	set_height(PlayerHeight);
	set_velocity(abfw::Vector2(0,0));
	set_on_platform(false);
}

void Player::Update(abfw::ControllerInputManagerVita& Input, std::vector<GameObject>& platforms)
{
	// equip the armour if the player has it
	if(has_armour() == true)
		set_colour(ArmourColour);

	// change the player's horizontal velocity using the left stick
	if(Input.left_stick_x_axis() == -1)
		set_velocity(abfw::Vector2(-HorizontalVelocity, velocity().y));

	if(Input.left_stick_x_axis() == 1)
		set_velocity(abfw::Vector2(HorizontalVelocity, velocity().y));

	if(Input.left_stick_x_axis() == 0)
		set_velocity(abfw::Vector2(0, velocity().y));

	// if the player isn't on a platform then make the player fall at a steadily increasing rate
	if(on_platform() == false)
		set_velocity(abfw::Vector2(velocity().x, velocity().y + VerticalAcceleration));
	
	// keep the player from exceeding their max velocity
	if(velocity().y > MaxVerticalVelocity)
		set_velocity(abfw::Vector2(velocity().x, MaxVerticalVelocity));
	
	// if the player is falling then check for when it collides with a platform
	if(velocity().y >= 0)
	{
		if(PlatformCollisionDetection(platforms) == true)
		{
			// stop the player from falling
			set_velocity(abfw::Vector2(velocity().x, 0));
			
			// reset the on platform variable so the player can jump again
			set_on_platform(true);
		}
	}
	
	// when the player isn't on a platform then set the on platform variable to false
	if(PlatformCollisionDetection(platforms) == false)
		set_on_platform(false);

	// check if the player is on a platform
	if(on_platform() == true)
	{
		// when the left stick is pushed up, move the player up and prevent them from jumping again
		if(Input.left_stick_y_axis() == -1)
		{
			set_velocity(abfw::Vector2(velocity().x, velocity().y - InitialVerticalVelocity));
			set_on_platform(false);
		}
	}

	// add the player's velocity to its position
	MoveDeltaX(velocity().x);
	MoveDeltaY(velocity().y);
}

bool Player::PlatformCollisionDetection(std::vector<GameObject>& platforms)
{	
	bool platform_hit = false;

	// loop through every platform
	for(std::vector<GameObject>::iterator current_platform = platforms.begin(); current_platform < platforms.end(); current_platform++)
	{
		// get the position of the top of the current platform
		float top_of_platform = current_platform->position().y - (HalfSize(current_platform->height())) - (HalfSize(height()));
		
		// check if the player is colliding with the current platform
		if(CollisionDetection(*current_platform) == true)
		{
			platform_hit = true;

			// if the player is falling then move them to the top of the platform
			if(velocity().y > 0)
				SetY(top_of_platform);
		}
	}

	return platform_hit;
}
