#ifndef PLAYER_H
#define PLAYER_H

#include <input/vita/controller_input_manager_vita.h>
#include <vector>

#include "gameobject.h"

// define player properties
const float InitialVerticalVelocity = 20.f, // speed that the player is moved up when they start their jump
			VerticalAcceleration = 1.5f, // the rate at which the player's downwards speed increases at a time
			MaxVerticalVelocity = 10.f, // the maximum speed a player can move downwards at a time
			HorizontalVelocity = 5.f, // the speed the player moves on the x axis
			PlayerWidth = 32.f, // width of player
			PlayerHeight = 32.f, // height of player
			PlayerHealth = 400.f, // how much health the player has
			PlayerZCoordinate = 0.1f;  // position of the player in the z axis
const UInt32 PlayerColour = 0xfffac664; // colour of the player
// define item properties
const float KeyWidth = 10.f,
			KeyHeight = 20.f,
			ArmourWidth = 32.f,
			ArmourHeight = 20.f,
			ArmourResistance = 0.5f,
			ObjectiveWidth = 25.f,
			ObjectiveHeight = 25.f;
const UInt32 KeyColour = 0xff00ffff,
			 ArmourColour = 0xff545454,
			 ObjectiveColour = 0xff0000ff;
				
class Player : public GameObject
{
 public:
	Player();  // initialises player variables
	void Update(abfw::ControllerInputManagerVita& Input, std::vector<GameObject>& platforms);
	// getters and setters
	inline void set_number_of_keys(const int number_of_keys) { number_of_keys_ = number_of_keys;}
	inline const int number_of_keys() const {return number_of_keys_; }
	inline void set_on_platform(const bool on_platform) { on_platform_ = on_platform; }
	inline const bool on_platform() const {return on_platform_; }
	inline void set_has_first_key(const bool has_first_key) { has_first_key_ = has_first_key; }
	inline const bool has_first_key() const {return has_first_key_; }
	inline void set_has_second_key(const bool has_second_key) { has_second_key_ = has_second_key; }
	inline const bool has_second_key() const {return has_second_key_; }
	inline void set_has_third_key(const bool has_third_key) { has_third_key_ = has_third_key; }
	inline const bool has_third_key() const {return has_third_key_; }
	inline void set_has_armour(const bool has_armour) { has_armour_ = has_armour; }
	inline const bool has_armour() const {return has_armour_; }
	inline void set_has_stick(const bool has_stick) { has_stick_ = has_stick; }
	inline const bool has_stick() const {return has_stick_; }
	inline void set_has_gun_upgrade(const bool has_gun_upgrade) { has_gun_upgrade_ = has_gun_upgrade; }
	inline const bool has_gun_upgrade() const {return has_gun_upgrade_; }
	inline void set_has_sword(const bool has_sword) { has_sword_ = has_sword; }
	inline const bool has_sword() const {return has_sword_; }
	bool PlatformCollisionDetection(std::vector<GameObject>& platforms); // checks if there is a platform below the player

 private:
	int number_of_keys_; // number of keys that the player has collected
	bool on_platform_, // wether or not player is on a platform
		 has_first_key_, // wether or not player has the first key
		 has_second_key_, // wether or not player has the second key
		 has_third_key_, // wether or not player has the third key
		 has_armour_, // wether or not player has the armour
		 has_stick_, // wether or not player has the stick
		 has_gun_upgrade_, // wether or not player has the gun upgrade
		 has_sword_; // wether or not player has the sword
};

#endif
