#include "weapon.h"

Weapon::Weapon()
{
	// initialise weapon properties
	set_colour(GunColour);
	set_width(GunWidth);
	set_height(WeaponHeight);
	set_orientation(Right);
	set_fire_rate(StandardFireRate);
	set_offset_length(GunOffset);
	set_can_fire(true);
	set_damage(GunDamage);
}

void Weapon::Aim(float stick_x_axis, float stick_y_axis)
{
	// use trigonometry to get the orientation of the right stick
	set_orientation(atan2(stick_y_axis, stick_x_axis));
	// rotate the weapon so that it's facing the same direction as the right stick
	set_rotation(orientation());
	// use trigonometry and the offset length to calculate the offsets of the weapon
	offset_.x = cos(orientation()) * offset_length();
	offset_.y = sin(orientation()) * offset_length();

	// change the z coordinate of the weapon based on the direction it's facing
	// to simulate the player turning around
	if(orientation() < Up || orientation() > Down)
		SetZ(BehindPlayer);
	else
		SetZ(FrontOfPlayer);
}

void Weapon::Melee(std::vector<GameObject>& enemies, int melee_type)
{
	// prevent the weapon from firing whilst in melee mode
	set_can_fire(false);

	// set the weapon properties to its stick settings
	if(melee_type == StickMelee)
	{
		set_colour(StickColour);
		set_width(StickWidth);
		set_offset_length(StickOffset);
		set_damage(StickDamage);
	}
	
	// set the weapon properties to its sword settings
	if(melee_type == SwordMelee)
	{
		set_colour(SwordColour);
		set_width(SwordWidth);
		set_offset_length(SwordOffset);
		set_damage(SwordDamage);
	}

	// loop through all the enemies
	std::vector<GameObject>::iterator enemy = enemies.begin();
	while(enemy != enemies.end())
	{
		// if an enemy is hit by the weapon, it takes damage and is knocked back
		if(CollisionDetection(*enemy) == true)
		{
			enemy->set_health(enemy->health() - damage());
			enemy->set_speed(EnemyKnockback);
		}
		else
			enemy->set_speed(EnemySpeed);

		// when an enemy's health reaches zero, delete the enemy
		if(enemy->health() < 0.1)
			enemies.erase(enemy);
		else
			enemy++;
	}
}

void Weapon::Ranged()
{
	// set the weapon properties to its ranged settings
	set_colour(GunColour);
	set_width(GunWidth);
	set_offset_length(GunOffset);
	set_can_fire(true);
	set_damage(GunDamage);
}

void Weapon::CreateProjectile(float xposition, float yposition, float direction)
{
	// create a new game object for the projectile
	GameObject projectile;
	// initialise projectile variables
	projectile.set_colour(ProjectileColour);
	projectile.set_width(ProjectileWidth);
	projectile.set_height(ProjectileHeight);
	projectile.set_speed(ProjectileSpeed);
	projectile.MoveTo(xposition, yposition, 0);
	projectile.set_orientation(direction);
	// add the new projectile to the vector of bullets
	Bullets.push_back(projectile);
}
