#ifndef WEAPON_H
#define WEAPON_H

#include <vector>

#include "gameobject.h"

// define directions
const float Up = -1.6f,
			Down = 1.6f,
			Left = 3.1f,
			Right = 0.f;
// define weapon properties
const UInt32 GunColour = 0xff000000,
			 StickColour = 0xff009bff,
			 SwordColour = 0xffc4c4c4;
const float GunWidth = 24.f,
			StickWidth = 36.f,
			SwordWidth = 48.f,
			WeaponHeight = 8.f,
			GunOffset = 16.f,
			StickOffset = 24.f,
			SwordOffset = 32.f,
			GunDamage = 1.f,
			StickDamage = 0.1f,
			SwordDamage = 5.f,
			FrontOfPlayer = 0.f,
			BehindPlayer = 0.2f,
			EnemyKnockback = -50.f;
const int StandardFireRate = 15,
		  ImprovedFireRate = 10,
		  StickMelee = 1,
		  SwordMelee = 2;
// define projectile properties
const UInt32 ProjectileColour = 0xffffffff;
const float ProjectileWidth = 5.f,
			ProjectileHeight = 5.f,
			ProjectileSpeed = 15.f;

class Weapon : public GameObject
{
 public:
	Weapon();  // initialises weapon variables
	std::vector<GameObject> Bullets;
	// getters and setters
	inline void set_offset(const abfw::Vector2& offset) { offset_ = offset; }
	inline const abfw::Vector2& offset() const {return offset_; }
	inline void set_offset_length(const float offset_length) { offset_length_ = offset_length; }
	inline const float offset_length() const {return offset_length_; }
	inline void set_damage(const float damage) { damage_ = damage; }
	inline const float damage() const {return damage_; }
	inline void set_fire_rate(const int fire_rate) { fire_rate_ = fire_rate;}
	inline const int fire_rate() const {return fire_rate_; }
	inline void set_can_fire(const bool can_fire) { can_fire_ = can_fire; }
	inline const bool can_fire() const {return can_fire_; }
	void Aim(float stick_x_axis, float stick_y_axis), // aim the weapon in the direction of the right stick
		 Melee(std::vector<GameObject>& enemies, int melee_type), // change to melee weapon
		 Ranged(), // change to ranged weapon
		 CreateProjectile(float xposition, float yposition, float direction);

 private:
	abfw::Vector2 offset_; // offset to place the weapon at so it rotates from its handle
	float offset_length_, // length of the offset
		  damage_; // damage the weapon inflicts on enemies
	int fire_rate_; // rate that projectiles are fired from the weapon
	bool can_fire_; // wether or not the weapon can currently fire
};

#endif
